#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include <string.h>
#include "neo.h"

void neo_command_port(int argc, char **argv)
{
  argc--;
  argv++;

  if (argc<1) {
    printf("Not enough arguments to port command.\n");
    return;
  }
  
  if (commatch(argv[0], "enable", "ena", "en", NULL)) {
    neo_command_port_sed(g, argc, argv);
  } else if (commatch(argv[0], "disable", "dis", NULL)) {
    neo_command_port_sed(g, argc, argv);
  } else if (commatch(argv[0], "status", "sta", "stat", NULL)) {
    neo_command_port_sed(g, argc, argv);
  } else if (commatch(argv[0], "search", NULL)) {
    neo_command_port_search(g, argc, argv);
  } else if (commatch(argv[0], "summary", "summ", "sum", "su", NULL)) {
    argc--; argv++;
    neo_command_device_summary(g, argc, argv);
  } else {
    printf("Invalid argument '%s' to port command.\n", argv[0]);
    return;
  }
}

void neo_command_port_sed(neo_global *g, int argc, char **argv)
{
  int command=0, ret, i, j;
  neo_location loc;
  neo_device *d;
#ifdef MIT
  char buff[1024];
#endif

  if (commatch(argv[0], "enable", "ena", "en", NULL)) {
    command=NEO_COMMAND_PORT_ENABLE;
  } else if (commatch(argv[0], "disable", "dis", NULL)) {
    command=NEO_COMMAND_PORT_DISABLE;
  } else if (commatch(argv[0], "status", "sta", "stat", NULL)) {
    command=NEO_COMMAND_PORT_STATUS;
  }
  
  argc--;
  argv++;
  
  if (argc>1) {
    printf("Sorry, only one location for now.\n");
    return;
  } else if (argc<1) {
    printf("Not enough arguments\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }

  if (neo_location_star_board(&loc)) {
    printf("You can not currently access multiple boards at a time.\n");
    return;
  }
  if (neo_location_star_port(&loc)) {
    printf("You can not currently access multiple ports at a time.\n");
    return;
  }
  if (!neo_location_has_port(&loc)) {
    printf("You must specify a port to access\n");
    return;
  }

  ret=neo_location_set_deviceconsts(&loc, g);
  if (ret) {
    return;
  }

  j=neo_location_num_hosts(&loc);
  if (j>1) {
    /* there's no reason we can't do this, it just seems like a bad idea to let the user do it */
    printf("You can not currently turn off ports on more than one device at a time.\n");
    return;
  }

  /* we'll write the code to do many anyway */
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);

    if (neo_device_has_boards(d) && !neo_location_has_board(&loc)) {
      printf("You must specify a board for '%s' since it is a %s and has boards\n",
	     neo_device_get_name(d), neo_device_get_devname(d));
      return;
    }
    if (!neo_device_has_boards(d) && neo_location_has_board(&loc)) {
      printf("You have specified a board for '%s' but it is a %s and has no boards\n",
	     neo_device_get_name(d), neo_device_get_devname(d));
      return;
    }

#ifdef MIT
    if (command==NEO_COMMAND_PORT_ENABLE) {
      sprintf(buff, "port enable request %s", neo_location_get_instr(&loc));
      neo_register(buff);
    } else if (command==NEO_COMMAND_PORT_DISABLE) {
      sprintf(buff, "port disable request %s", neo_location_get_instr(&loc));
      neo_register(buff);
    }
#endif
    
    ret=neo_device_command_port_sed(d, &loc, command);
      
    if (ret==NEO_COMMAND_PORT_IS_ENABLED) {
      printf("%s enabled\n", neo_location_get_instr(&loc));
    } else if (ret==NEO_COMMAND_PORT_IS_DISABLED) {
      printf("%s disabled\n", neo_location_get_instr(&loc));
    } else if (ret==NEO_COMMAND_PORT_IS_UNKNOWN) {
      printf("Unknown port status for %s\n", neo_location_get_instr(&loc));
    } else if (ret==NEO_COMMAND_PORT_IS_SILENTERR) {
      /* do nothing */
    } else if (ret==-1) {
      if (neo_error==NEO_ERR_CONTACT) {
	printf("Could not contact device %s\n", neo_device_get_name(d));
      } else {
	printf("Error talking to device %s\n", neo_device_get_name(d));
      }
    }

#ifdef MIT
    if (command==NEO_COMMAND_PORT_ENABLE && ret==NEO_COMMAND_PORT_IS_ENABLED) {
      sprintf(buff, "port enable success %s", neo_location_get_instr(&loc));
      neo_register(buff);
    } else if (command==NEO_COMMAND_PORT_DISABLE && ret==NEO_COMMAND_PORT_IS_DISABLED) {
      sprintf(buff, "port disable success %s", neo_location_get_instr(&loc));
      neo_register(buff);
    }
#endif

    neo_location_free(&loc);
  }
}

void neo_command_arpfind(int argc, char **argv)
{
  neo_searchable s;
  neo_macaddr m;
  neo_location loc;
  neo_device *d;
  int ret;
  char buff[1024];
  char *ipaddr;
#ifdef MIT
  char *ptr, router[300];
#endif
  int i, j;

  argc--;
  argv++;

#ifdef MIT
  if (argc < 1) {
    printf("Usage: arpfind <host> [<location>]\n");
    return;
  }
#else
  if (argc != 2) {
    printf("Usage: arpfind <host> <location>\n");
    return;
  }
#endif
  
  /* translate the item */
  neo_searchable_create_from_string(&s, argv[0]);
  
  if (neo_searchable_primary_is_macaddr(&s)) {
    printf("You must supply an IP address or hostname\n");
    neo_searchable_free(&s);
    return;
  }

  /* we want the ip addr if we have the host */
  if (neo_searchable_primary_is_hostname(&s)) {
    ret=neo_searchable_expand_hostip(&s);
    if (ret) {
      printf("Could not find ip address for host %s\n", argv[0]);
      neo_searchable_free(&s);
      return;
    }
  }

  ipaddr=neo_searchable_get_ipaddr(&s);

#ifdef MIT
  if (argc==1) {
    /* here we will figure out which MITnet router to ask */
    /* in the future we'll want to look at something like -k to get a list of hosts */
    strcpy(router, ipaddr);
    ptr=strchr(router, '.');
    if (!ptr) {
      printf("Invalid IP address %s\n", ipaddr);
      neo_searchable_free(&s);
      return;
    }
    ptr=strchr(ptr+1, '.');
    if (!ptr) {
      printf("Invalid IP address %s\n", ipaddr);
      neo_searchable_free(&s);
      return;
    }
    if (!strncmp(ptr-2, ".7.", 3)) {
      /* one more for 18.7.x.y */
      ptr=strchr(ptr+1, '.');
      if (!ptr) {
	printf("Invalid IP address %s\n", ipaddr);
	neo_searchable_free(&s);
	return;
      }
      router[ptr-router]='\0';
      strcat(router, ".1");
    } else {
      router[ptr-router]='\0';
      strcat(router, ".0.1");
    }
    ret=neo_location_create_from_string(&loc, router);
    
  } else if (argc==2) {
    ret=neo_location_create_from_string(&loc, argv[1]);
    if (ret) {
      printf("Could not parse location %s\n", argv[1]);
      neo_searchable_free(&s);
      return;
    }
  }
#else
  ret=neo_location_create_from_string(&loc, argv[1]);
  if (ret) {
    printf("Could not parse location %s\n", argv[1]);
    neo_searchable_free(&s);
    return;
  }
#endif
  
  j=neo_location_num_hosts(&loc);
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);

    ret=neo_function_arpfind(&m, ipaddr, neo_device_get_name(d));
    if (!ret) {
      neo_macaddr_to_string(&m, buff);
      printf("%s says %s is %s\n", neo_device_get_name(d), ipaddr, buff);
    } else {
      printf("arpfind: %s can not find entry for %s\n", neo_device_get_name(d), ipaddr);
    }
  }
  neo_location_free(&loc);
  neo_searchable_free(&s);
}

void neo_command_board(int argc, char **argv)
{
  argc--;
  argv++;

  if (argc<1) {
    printf("Not enough arguments to board command.\n");
    return;
  }

  if (commatch(argv[0], "summary", "summ", "sum", "su", NULL)) {
    argc--; argv++;
    neo_command_device_summary(g, argc, argv);
  } else {
    printf("Invalid argument '%s' to board command.\n", argv[0]);
    return;
  }
}

void summary_print_boards(neo_boardlist *s);

void summary_print_ports(neo_portlist *s);

void neo_command_device(int argc, char **argv)
{
  argc--;
  argv++;

  if (argc<1) {
    printf("Not enough arguments for 'device'.\n");
    return;
  }
  
  if (commatch(argv[0], "type", NULL)) {
    neo_command_device_type(--argc, ++argv);
  } else if (commatch(argv[0], "info", NULL)) {
    neo_command_device_info(--argc, ++argv);
  } else if (commatch(argv[0], "set", NULL)) {
    neo_command_device_set(--argc, ++argv);
  } else if (commatch(argv[0], "summary", "summ", "sum", "su", NULL)) {
    neo_command_device_summary(g, --argc, ++argv);
  } else if (commatch(argv[0], "reset", NULL)) {
    neo_command_device_reset(--argc, ++argv);
  } else if (commatch(argv[0], "download", "downl", NULL)) {
    neo_command_device_load("download", --argc, ++argv);
  } else if (commatch(argv[0], "upload", NULL)) {
    neo_command_device_load("upload", --argc, ++argv);
  } else if (commatch(argv[0], "uplink", NULL)) {
    neo_command_device_uplink(--argc, ++argv);
  } else if (commatch(argv[0], "capabilities", "cap", "capa", "capb", NULL)) {
    neo_command_device_capabilities(--argc, ++argv);
  } else if (commatch(argv[0], "?")) {
    printf("device subcommands:\n");
    printf("   type\n");
    printf("   info\n");
    printf("   uplink\n");
    printf("   capabilities\n");
    printf("   set\n");
    printf("   summary\n");
    printf("   reset\n");
    printf("   download\n");
    printf("   upload\n");
  } else {
    printf("Unknown 'device' subcommand '%s'\n", argv[0]);
    return;
  }
}

void neo_command_device_type(int argc, char **argv)
{
  int i,j, ret;
  neo_location loc;
  neo_device *d;

  if (argc<1) {
    printf("Not enough arguments for locate.\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }

  ret=neo_location_set_deviceconsts(&loc, g);
  if (ret) printf("Error setting device constants\n");
  
  j=neo_location_num_hosts(&loc);
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    if (!neo_device_is_probed(d)) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
    } else {
      printf("%s: %s\n", neo_device_get_name(d), neo_device_get_devname(d));
    }
  }

  neo_location_free(&loc);
}

void neo_command_device_capabilities(int argc, char **argv)
{
  int ret, i, j;
  neo_location loc;
  neo_device *d;
  
  if (argc!=1) {
    printf("You must specifiy a single location\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }

  ret=neo_location_set_deviceconsts(&loc, g);
  if (ret) printf("Error setting device constants\n");

  j=neo_location_num_hosts(&loc);
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    if (!neo_device_is_probed(d)) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
    } else {
      neo_device_print_capabilities(d);
    }
  }

  neo_location_free(&loc);
}

void neo_command_device_set(int argc, char **argv)
{
  int i, j;
  neo_location loc;
  neo_device *d;
  int ret;
  char val[1024];
  char *obj;

  if (argc<3) {
    printf("Not enough arguments for set.\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }

  if (!strcasecmp(argv[1], "system")) {
    if (!strcasecmp(argv[2], "contact")) {
      obj=".1.3.6.1.2.1.1.4.0";
    } else if (!strcasecmp(argv[2], "name")) {
      obj=".1.3.6.1.2.1.1.5.0";
    } else if (!strcasecmp(argv[2], "location")) {
      obj=".1.3.6.1.2.1.1.6.0";
    } else {
      printf("Unknown system object.\n");
      return;
    }
    if (argc<4) {
      printf("No value specified.\n");
      return;
    }
    strcpy(val, "");
    for (i=0; i<argc-3; i++) {
      if (i>0) strcat(val, " ");
      strcat(val, argv[i+3]);
    }
  } else if (!strncmp(argv[1], ".1.3.6", 6)) {
    obj=argv[1];
    
    if (argc<3) {
      printf("No value specified.\n");
      return;
    }
    strcpy(val, "");
    for (i=0; i<argc-2; i++) {
      if (i>0) strcat(val, " ");
      strcat(val, argv[i+2]);
    }
  } else {
    printf("Unknown object.\n");
    return;
  }

  neo_location_set_deviceconsts(&loc, g); /* do we really need this ? */
  
  j=neo_location_num_hosts(&loc);
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    if (!neo_device_is_probed(d)) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
    } else {
      neo_snmp_set_by_variable(neo_device_get_name(d), obj, "string", val);
      printf("Value set.\n");
    }
  }
  
  neo_location_free(&loc);
}

void neo_command_device_reset(int argc, char **argv)
{
  int i, j, ret;
  neo_device *d;
  neo_location loc;
  
  if (argc<1) {
    printf("You must supply a location.\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }

  neo_location_set_deviceconsts(&loc, g);

  j=neo_location_num_hosts(&loc);
  if (j>1) {
    printf("WARNING! You have specified more than one device to reset.\n");
    if (!are_you_sure(NULL)) return;
  }
  
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    if (!neo_device_is_probed(d)) {
      printf("Could not contact host %s.\n", neo_device_get_name(d));
    } else {
      neo_device_command_reset(d);
    }
  }
  neo_location_free(&loc);
}

void neo_command_device_uplink(int argc, char **argv)
{
  neo_locationlist ans;
  int ret, y, x;

  if (argc!=1) {
    printf("Please supply one hostname\n");
    return;
  }
  
  neo_locationlist_create(&ans);
  ret=neo_function_get_uplink(&ans, argv[0]);
  if (ret) {
    printf("device_uplink: error getting uplink\n");
    return;
  }
  
  y=neo_locationlist_get_size(&ans);
  if (y>0) {
    for (x=0; x<y; x++) {
      printf("Uplink found on %s\n", neo_location_get_instr(neo_locationlist_get_element(&ans, x)));
    }
  }
  neo_locationlist_free_all(&ans);
}

void neo_command_exec(int argc, char **argv)
{
  int i;
  
  argc--;
  argv++;

  if (argc<1) {
    printf("Not enough arguments to the 'exec' command.\n");
    return;
  }
	   
  i=fork();

  if (i==0) {
    execvp(argv[0], argv);
  } else {
    wait3(NULL, 0, NULL);
  }
  return;
}

void neo_command_device_info(int argc, char **argv)
{
  neo_location loc;
  neo_device *d;
  int type, i, j, ret;

  if (argc<1) {
    printf("Not enough arguments, you must supply a location.\n");
    return;
  }

  if (argc==1) {
    /* do 'device info system' for backwards compat */
    neo_command_device_info_system(argc, argv);
    return;
  }
      
  if (!strcmp(argv[0], "system")) {
    neo_command_device_info_system(--argc, ++argv);
    return;
  } else if (!strcmp(argv[0], "power")) {
    type=NEO_COMMAND_DEVICE_INFO_POWER;
  } else if (!strcmp(argv[0], "weather")) {
    type=NEO_COMMAND_DEVICE_INFO_WEATHER;
  } else if (!strcmp(argv[0], "cpu")) {
    type=NEO_COMMAND_DEVICE_INFO_CPU;
  } else if (!strcmp(argv[0], "printer")) {
    type=NEO_COMMAND_DEVICE_INFO_PRINTER;
  } else if (!strcmp(argv[0], "ups")) {
    type=NEO_COMMAND_DEVICE_INFO_UPS;
  } else {
    printf("Unknown subcommand '%s'.\n", argv[0]);
    return;
  }

  /* loop on devices */
  ret=neo_location_create_from_string(&loc, argv[1]);
  if (ret) {
    printf("Could not parse location %s\n", argv[1]);
    return;
  }
  neo_location_set_deviceconsts(&loc, g);
  
  j=neo_location_num_hosts(&loc);
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    if (!neo_device_is_probed(d)) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
    } else {
      neo_device_command_info(d, type);
    }
  }
}

void neo_command_device_info_system(int argc, char **argv)
{
  int i,j;
  int days, hours, minutes;
  long up;
  neo_location loc;
  neo_device *d;
  int ret;
  char buff[1024];

  if (argc<1) {
    printf("Not enough arguments, you must supply a location.\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }
  neo_location_set_deviceconsts(&loc, g);
  
  j=neo_location_num_hosts(&loc);
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    if (!neo_device_is_probed(d)) {
      printf("Could not contact device %s\n", neo_device_get_name(d));
    } else {
      printf("%s\n", neo_device_get_name(d));
      printf("  Device type: %s\n", neo_device_get_devname(d));

      ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.1.4.0", buff);
      if (ret) {
	printf("Error getting sysContact.\n");
      } else {
	printf("  Contact    : %s\n", buff);
      }

      ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.1.5.0", buff);
      if (ret) {
	printf("Error getting sysName.\n");
      } else {
	printf("  Name       : %s\n", buff);
      }

      ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.1.6.0", buff);
      if (ret) {
	printf("Error getting sysLocation.\n");
      } else {
	printf("  Location   : %s\n", buff);
      }

      ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.1.3.0", buff);
      if (ret) {
	printf("Error getting sysUpTime.\n");
      } else {
	buff[strlen(buff)-2]='\0'; /* chop off last two digits */
	up=atol(buff);
	days=up/86400;
	up-=days*86400;
	hours=up/3600;
	up-=hours*3600;
	minutes=up/60;
	up-=minutes*60;
	printf("  Uptime     : %i days %2.2i:%2.2i:%2.2li\n", days, hours, minutes, up);
      }

      ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.1.2.0", buff);
      if (ret) {
	printf("Error getting sysObjectID.\n");
      } else {
	printf("  ObjectID   : %s\n", buff);
      }

      ret=neo_snmp_request_by_variable(neo_device_get_name(d), ".1.3.6.1.2.1.1.1.0", buff);
      if (ret) {
	printf("Error getting sysDescr.\n");
      } else {
	printf("  Descr      : %s\n", buff);
      }

    }
  }

  neo_location_free(&loc);
}

void neo_command_device_load(char *op, int argc, char **argv)
{
  neo_location loc;
  neo_device *d;
  int ret, type, direction;
  char *typestr, *tftphost, *filename;
  
  if (argc!=4) {
    printf("Usage: %s image|config <location> <tftphost> <filename>\n", op);
    return;
  }

  typestr=argv[0];
  tftphost=argv[2];
  filename=argv[3];

  /* set the type */
  if (!strcmp(typestr, "image")) {
    type=NEO_COMMAND_DEVICE_LOAD_IMAGE;
  } else if (!strcmp(typestr, "config")) {
    type=NEO_COMMAND_DEVICE_LOAD_CONFIG;
  } else {
    printf("Usage: %s image|config <location> <tftphost> <filename>\n", op);
    return;
  }

  /* set the direction */
  if (!strcmp(op, "download")) {
    direction=NEO_COMMAND_DEVICE_LOAD_DOWNLOAD;
  } else if (!strcmp(op, "upload")) {
    direction=NEO_COMMAND_DEVICE_LOAD_UPLOAD;
  } else {
    printf("Internal error\n");
    neo_location_free(&loc);
    return;
  }
  
  ret=neo_location_create_from_string(&loc, argv[1]);
  if (ret) {
    printf("Could not parse location %s\n", argv[1]);
    return;
  }
  if (!neo_location_is_simplehost(&loc)) {
    printf("Please only specify one host for this operation\n");
    neo_location_free(&loc);
    return;
  }

  neo_location_set_deviceconsts(&loc, g);

  d=neo_location_get_first_device(&loc);
  if (!neo_device_is_probed(d)) {
    printf("Could not contact device %s\n", neo_device_get_name(d));
  } else {
    ret=neo_device_command_load(d, &loc, type, direction, tftphost, filename);
    if (ret) {
      printf("Error doing load\n");
    }
  }
  neo_location_free(&loc);
}

void neo_command_print(int argc, char **argv)
{
  argc--;
  argv++;

  if (argc>1) {
    printf("Too many arguments to the 'print' command\n");
    return;
  }

  if (argc==0) {
    printf("writecom = '%s'\n", neo_global_get_writecom(g));
    printf("readcom = '%s'\n", neo_global_get_readcom(g));
    printf("keyfile = '%s'\n", neo_global_get_keyfile(g));
    printf("statsdelay = %i\n", neo_global_get_statsdelay(g));
    printf("timeout = %i\n", neo_global_get_timeout(g));
    printf("macmode = '%s'\n", neo_global_get_macmode_string(g));
    printf("version = '%s'\n", neo_global_get_version(g));
    printf("burst = %i\n", neo_global_get_burst(g));
    printf("verbose = %s\n", neo_global_is_verbose(g) ? "on" : "off");
    printf("debug = %s\n", neo_global_is_debug(g) ? "on" : "off");
    return;
  }
  
  if (!strcmp(argv[0], "writecom")) {
    printf("writecom = '%s'\n", neo_global_get_writecom(g));
  } else if (!strcmp(argv[0], "readcom")) {
    printf("readcom = '%s'\n", neo_global_get_readcom(g));
  } else if (!strcmp(argv[0], "burst")) {
    printf("burst = %i\n", neo_global_get_burst(g));
  } else if (!strcmp(argv[0], "keyfile")) {
    printf("keyfile = '%s'\n", neo_global_get_keyfile(g));
  } else if (!strcmp(argv[0], "statsdelay")) {
    printf("statsdelay = %i\n", neo_global_get_statsdelay(g));
  } else if (!strcmp(argv[0], "timeout")) {
    printf("timeout = %i\n", neo_global_get_timeout(g));
  } else if (!strcmp(argv[0], "macmode")) {
    printf("macmode = '%s'\n", neo_global_get_macmode_string(g));
  } else if (!strcmp(argv[0], "version")) {
    printf("version = '%s'\n", neo_global_get_version(g));
  } else if (!strcmp(argv[0], "verbose")) {
    printf("verbose = %s\n", neo_global_is_verbose(g) ? "on" : "off");
  } else if (!strcmp(argv[0], "debug")) {
    printf("debug = %s\n", neo_global_is_debug(g) ? "on" : "off");
  } else {
    printf("Unknown variable '%s'.\n", argv[0]);
  }
}

void neo_command_set(int argc, char **argv)
{
  char buff[1024];
  argc--;
  argv++;

  if (argc>2) {
    printf("Too many arguments to the 'set' command\n");
    return;
  }

  if (argc<1) {
    printf("Not enough arguments to the 'set' command\n");
    return;
  }
  
  if (!strcmp(argv[0], "writecom")) {
    if (argc==2) {
      neo_global_set_writecom(g, argv[1]);
      return;
    } else if (argc==1) {
      if (get_pass("Write community: ", buff, 1024)) {
	return;
      }
      neo_global_set_writecom(g, buff);
      return;
    }
  } else if (!strcmp(argv[0], "readcom")) {
    if (argc==2) {
      neo_global_set_readcom(g, argv[1]);
      return;
    } else if (argc==1) {
      if (get_pass("Read community: ", buff, 1024)) {
	return;
      }
      neo_global_set_readcom(g, buff);
      return;
    }
  }

  if (argc<2) {
    printf("Not enough arguments to the 'set' command\n");
    return;
  }

  if (!strcmp(argv[0], "burst")) { /* check for too few args */
    neo_global_set_burst(g, atoi(argv[1]));
  } else if (!strcmp(argv[0], "keyfile")) {  /* check for too few args */
    neo_global_set_keyfile(g, argv[1]);
  } else if (!strcmp(argv[0], "statsdelay")) {
    neo_global_set_statsdelay(g, atoi(argv[1]));
  } else if (!strcmp(argv[0], "timeout")) {
    neo_global_set_timeout(g, atoi(argv[1]));
  } else if (!strcmp(argv[0], "macmode")) {
    if (neo_global_set_macmode_from_string(g, argv[1])<0) {
      printf("Invalid mode for mac format.  Please choose from 'standard',\n");
      printf("'cisco', or 'dash'.\n");
    }
  } else if (!strcmp(argv[0], "version")) {
    printf("You may not change the variable 'version'.\n");
  } else if (!strcmp(argv[0], "debug")) {
    if (!strcmp(argv[1], "on")) {
      neo_global_set_debug(g);
    } else if (!strcmp(argv[1], "off")) {
      neo_global_set_nodebug(g);
    } else {
      printf("Invalid setting for 'debug' variable.\n");
    }
  } else if (!strcmp(argv[0], "verbose")) {
    if (!strcmp(argv[1], "on")) {
      neo_global_set_verbose(g);
    } else if (!strcmp(argv[1], "off")) {
      neo_global_set_noverbose(g);
    } else {
      printf("Invalid setting for 'verbose' variable.\n");
    }
  } else {
    printf("Unknown variable '%s'.\n", argv[0]);
  }

}

void neo_command_port_search(neo_global *g, int argc, char **argv)
{
  int ret, i, j;
  neo_location loc;
  neo_device *d;

  argc--;
  argv++;

  if (argc<1) {
    printf("Not enough arguments to port search command.\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }

  if (neo_location_star_board(&loc)) {
    printf("You can not currently search multiple boards at a time.\n");
    return;
  }
  if (neo_location_star_port(&loc)) {
    printf("You can not currently search multiple ports at a time.\n");
    return;
  }

  /* we've removed this because on things like the wireless AP's
   * you don't want to give a port number to search.
   * the device-specific functions are now responsbile for this
   * checking
   */
  /*
  if (!neo_location_has_port(&loc)) {
    printf("You must specify a port to search.\n");
    return;
  }
  */
  neo_location_set_deviceconsts(&loc, g);

  j=neo_location_num_hosts(&loc);
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    ret=neo_device_command_port_search(d, &loc);
  }

  neo_location_free(&loc);
}

void neo_command_device_summary(neo_global *g, int argc, char **argv)
{
  neo_location loc;
  neo_device *d;
  neo_list summary;
  int ret, i, j;
  
  if (argc<1) {
    printf("You must supply a location.\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }

  neo_location_set_deviceconsts(&loc, g);

  j=neo_location_num_hosts(&loc);
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    if (!neo_device_is_probed(d)) {
      printf("Could not contact host %s.\n", neo_device_get_name(d));
    } else {
      neo_list_create(&summary);
      ret=neo_device_command_summary(d, &loc, &summary);
      if (ret<0) {
	printf("Could not get summary for %s which is a %s\n", neo_device_get_name(d), neo_device_get_devname(d));
      } else {
	if (ret==NEO_TYPE_BOARDLIST) {
	  summary_print_boards((neo_boardlist *) &summary);
	} else if (ret==NEO_TYPE_PORTLIST) {
	  summary_print_ports((neo_portlist *) &summary);
	} else {
	  printf("Internal error generating summary for this device.\n");
	}
      }
      if (ret==NEO_TYPE_BOARDLIST) {
	neo_boardlist_free_all(&summary);
      } else if (ret==NEO_TYPE_PORTLIST) {
	neo_portlist_free_all(&summary);
      } else {
	neo_list_free_simple(&summary);
      }
    }
  }
  neo_location_free(&loc);
}

void summary_print_boards(neo_boardlist *s)
{
  int i, j;
  neo_board *b;

  printf("Board summary:\n");
  printf(" b type               sg   ul ultype     \n");  
  printf("-----------------------------------------\n");
  j=neo_boardlist_get_size(s);
  for (i=0; i<j; i++) {
    b=neo_boardlist_get_element(s, i);
    printf("%2i %-17.17s ", neo_board_get_number(b), neo_board_get_descr(b));

    if (!neo_board_is_present(b)) {
      printf("\n");
      continue;
    }
    
    printf("  %1i ", neo_board_get_segment(b));

    if (neo_board_link_is_up(b)) {
      printf("%4i ", neo_board_get_speed(b)/1000000);
    } else if (neo_board_link_is_down(b)) {
      printf("     ");
    } else if (neo_board_link_is_unknown(b)){
      printf(" ??? ");
    }

    printf("%-10.10s ", neo_board_get_uplinkdescr(b));

    printf("\n");
  }
}

void summary_print_ports(neo_portlist *s)
{
  int i, j;
  neo_port *p;
  printf("Port summary:\n");
  printf(" p type          u  lnk adm ap\n");  
  printf("------------------------------\n");

  j=neo_portlist_get_size(s);
  for (i=0; i<j; i++) {
    p=neo_portlist_get_element(s, i);
    printf("%2i %-13.13s ", neo_port_get_number(p), neo_port_get_descr(p));
    if (neo_port_get_uplink(p)) {
      printf("* ");
    } else {
      printf("  ");
    }
    if (neo_port_link_is_up(p) || neo_port_link_is_autopart(p)) {
      printf("%4i ", neo_port_get_speed(p)/1000000);
    } else if (neo_port_link_is_down(p)) {
      printf("   - ");
    } else {
      printf("???? ");
    }
    if (neo_port_get_admin(p) == 1) {
      printf(" On ");
    } else if (neo_port_get_admin(p) == 0) {
      printf("Off ");
    } else {
      printf("??? ");
    }

    if (neo_port_link_is_autopart(p)) {
      printf("!! ");
    }
    printf("\n");
  }
}

void neo_command_hostinfo(int argc, char **argv)
{
  struct hostent *he;
  struct sockaddr_in addr;
  char answer[PACKETSZ], buff[1024];
  int ret, i, len;
  const HEADER *hp;
  char *foo;

  argc--;
  argv++;

  if (argc<1) {
    printf("Not enough arguments to the 'hostinfo' command.\n");
    return;
  }

  /* printf("Desired host:   %s\n", argv[0]); */

  he=gethostbyname(argv[0]);
  if (!he) {
    printf("Could not resolve hostname '%s'.\n", argv[0]);
    return;
  }
  memcpy(&(addr.sin_addr.s_addr), he->h_addr, he->h_length);
  he=gethostbyaddr((char *) &(addr.sin_addr), sizeof(addr.sin_addr), AF_INET);

  if (!he) {
    printf("Could not resolve hostname '%s'.\n", argv[0]);
    return;
  }
  printf("Official name:  %s\n", he->h_name);
  printf("Host address:   %s\n", inet_ntoa(addr.sin_addr));


  /* query for hinfo */
  ret=res_query(he->h_name, C_IN, T_HINFO, answer, sizeof(answer));
  hp=(HEADER *) answer;
  if (hp->ancount==0) return; /* no hostinfo to return */
  if (ret<0 || hp->rcode != NOERROR) {
    perror("Error in res_query");
    return;
  }

  /* see RFC 1035 */
  
  /* skip through the question section */
  foo=answer+HFIXEDSZ;
  for (i=0; i<ntohs(hp->qdcount); i++) {
    foo=strchr(foo, '\0'); /* this is sort of cheating */
    if (!foo) {
      printf("Error.\n");
      return;
    }
    foo+=5; /* 1 + 4 */
  }
  
  /* now the answers */
  for (i=0; i<ntohs(hp->ancount); i++) {
    if ((((unsigned char) foo[0]) & 0xc0) == 0xc0) {
      /* it's compressed format */
      /* just skip the next byte */
      foo+=2;
    } else {
      foo+=((unsigned char) foo[0])+1;
    }
    
    if (((unsigned char) foo[1]) != 13) {
      /* it's not hinfo, so figure out where the next answer starts */
      foo+=2+2+4; /* this + type + ttl */
      /* now get the length */
      len=((unsigned char)foo[0])<<8;
      len+=(unsigned char)foo[1];
      foo+=2+len;
      continue;
    }
    foo+=2;
    
    if (((unsigned char) foo[1]) != 1) {
      printf("Unexpected type field in DNS response.\n");
      return;
    }
    foo+=2;
    
    /* skip the ttl */
    foo+=4;
    
    /* skip the length for now */
    foo+=2;
    
    len=(unsigned char) foo[0];
    foo++;
    strncpy(buff, foo, len);
    buff[len]='\0';
    printf("Host CPU:       %s\n", buff);
    foo+=len;
    
    len=(unsigned char) foo[0];
    foo++;
    strncpy(buff, foo, len);
    buff[len]='\0';
    printf("Host OS:        %s\n", buff);
  }
}

void neo_command_location(int argc, char **argv)
{
  argc--;
  argv++;

  if (argc<1) {
    printf("Not enough arguments for the 'location' command.\n");
    return;
  }

  if (commatch(argv[0], "print", "pr", NULL)) {
    neo_command_location_print(--argc, ++argv);
  } else {
    printf("Unknown subcommand '%s' to location command.\n", argv[0]);
    return;
  }
  return;
}

void neo_command_location_print(int argc, char **argv)
{
  int ret, i, j;
  neo_location loc;
  
  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    return;
  }

  j=neo_location_num_hosts(&loc);
  if (j==0) {
    printf("No devices in location %s.\n", argv[0]);
    return;
  }

  printf("Devices (%i) are:\n", j);
  for (i=0; i<j; i++) {
    printf("  %s\n", neo_device_get_name(neo_location_get_device_n(&loc, i)));
  }
  return;
}

void neo_command_locate(int argc, char **argv)
{
  int verbose, ret, i, j, findcount, uplink;
  neo_searchable s;
  neo_location loc;
  neo_locationlist ans;
  neo_device *d;

  argc--;
  argv++;

  verbose=0;
  uplink=0;
  while (argc>2) {
    if (!strcmp(argv[0], "-v")) {
      verbose=1;
      argv++;
      argc--;
    } else if (!strcmp(argv[0], "-u")) {
      uplink=1;
      argv++;
      argc--;
    } else {
      printf("Invalid argument to the locate.\n");
      return;
    }
  }
      
  if (argc!=2) {
    printf("Not enough arguments for locate.\n");
    return;
  }

  neo_searchable_create_from_string(&s, argv[0]);

  if (!neo_searchable_primary_is_macaddr(&s)) {
    printf("You must specify a macaddress for searching\n");
    neo_searchable_free(&s);
    return;
  }
  
  argv++;
  argc--;

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Could not parse location %s\n", argv[0]);
    neo_searchable_free(&s);
    return;
  }

  if (verbose) printf("Probing devices ...\n");
  neo_location_set_deviceconsts(&loc, g);

  findcount=0;
  j=neo_location_num_hosts(&loc);
  if (j==0) printf("No devices selected\n");
  for (i=0; i<j; i++) {
    d=neo_location_get_device_n(&loc, i);
    if (!neo_device_is_probed(d)) {
      if (verbose) printf("Skipping device %s\n", neo_device_get_name(d));
    } else {
      if (verbose) printf("Searching %s ...\n", neo_device_get_name(d));
      neo_locationlist_create(&ans);
      ret=neo_device_command_locate(d, &s, &ans, uplink);
      if (ret<0) {
	if (neo_error==NEO_ERR_INCAPABLE) {
	  printf("Incapable of doing locates on %s which is a %s\n", neo_device_get_name(d), neo_device_get_devname(d));
	} else if (neo_error==NEO_ERR_CONTACT) {
	  printf("Could not contact device %s\n", neo_device_get_name(d));
	} else {
	  printf("Error doing locates on %s\n", neo_device_get_name(d));
	}
      } else {
	int x, y;
	y=neo_locationlist_get_size(&ans);
	findcount+=y;
	if (y>0) {
	  for (x=0; x<y; x++) {
	    printf("Found on %s\n", neo_location_get_instr(neo_locationlist_get_element(&ans, x)));
	  }
	}
      }
      neo_locationlist_free_all(&ans);
    }
  }

  if (findcount==0) {
    printf("Not found\n");
  }

  if (verbose && findcount) {
    printf("%i locations found\n", findcount);
  }

  neo_location_free(&loc);
  neo_searchable_free(&s);
}

void neo_command_stats(int argc, char **argv)
{
  int ret, j;
  neo_location loc;
  neo_device *d;
  neo_list list;
    
  argc--;
  argv++;

  if (argc<1) {
    printf("Not enough arguments for stats.\n");
    return;
  }

  ret=neo_location_create_from_string(&loc, argv[0]);
  if (ret) {
    printf("Error parsing location\n");
    return;
  }

  printf("Probing devices ...\n");
  neo_location_set_deviceconsts(&loc, g);

  j=neo_location_num_hosts(&loc);
  if (j==1) {
    /* one device */
    d=neo_location_get_first_device(&loc);

    if (!neo_device_is_probed(d)) {
      return;
    }

    if (!neo_device_has_boards(d) && neo_location_has_board(&loc)) {
      printf("%s is a %s and has no boards\n", neo_device_get_name(d), neo_device_get_devname(d));
      neo_location_free(&loc);
      return;
    }

    neo_list_create(&list);
    ret=neo_device_command_stats(d, &loc, &list); /* JUST ASSUMING PORT LIST FOR THE MOMENT */
    if (ret==NEO_TYPE_PORTLIST) {
      neo_command_stats_print_portstats(&list);
      neo_portlist_free_all(&list);
    } else if (ret==NEO_TYPE_BOARDLIST) {
      neo_command_stats_print_boardstats(&list);
      neo_boardlist_free_all(&list);
    } else {
      neo_list_free_simple(&list);
    }
  } else if (j>1) {
    /* many devices */
    
  } else {
    printf("No devices found to search\n");
    neo_location_free(&loc);
    return;
  }

}

void neo_command_stats_print_portstats(neo_portlist *pl)
{
  int i, j;
  neo_port *p;
  neo_stats *s;

  printf("Port statistics:\n");
  printf(" p type          u  lnk adm ap   kbs   ikbs   okbs   pps  ipps  opps ierps oerps\n");  
  printf("--------------------------------------------------------------------------------\n");
  j=neo_portlist_get_size(pl);
  for (i=0; i<j; i++) {
    p=neo_portlist_get_element(pl, i);
    s=neo_port_get_stats(p);
    printf("%2i %-13.13s ", neo_port_get_number(p), neo_port_get_descr(p));
    if (neo_port_get_uplink(p)) {
      printf("* ");
    } else {
      printf("  ");
    }
    if (neo_port_link_is_up(p) || neo_port_link_is_autopart(p)) {
      printf("%4i ", neo_port_get_speed(p)/1000000);
    } else if (neo_port_link_is_down(p)) {
      printf("   - ");
    } else {
      printf(" ??? ");
    }
    if (neo_port_get_admin(p) == 1) {
      printf(" On ");
    } else if (neo_port_get_admin(p) == 0) {
      printf("Off ");
    } else {
      printf("??? ");
    }

    if (neo_port_link_is_autopart(p)) {
      printf("!! ");
    }

    if (neo_stats_get_generic(s, NEO_S_TOTALKBS, NEO_S_ERROR)) {
      printf("     ???");
    } else if (neo_stats_get_generic(s, NEO_S_TOTALKBS, NEO_S_UNAVAIL)) {
      printf("       -");
    } else {
      printf("  %6i", neo_stats_get_generic(s, NEO_S_TOTALKBS, NEO_S_INT));
    }
    
    if (neo_stats_get_generic(s, NEO_S_INKBS, NEO_S_ERROR)) {
      printf("    ???");
    } else if (neo_stats_get_generic(s, NEO_S_INKBS, NEO_S_UNAVAIL)) {
      printf("      -");
    } else {
      printf(" %6i", neo_stats_get_generic(s, NEO_S_INKBS, NEO_S_INT));
    }
    
    if (neo_stats_get_generic(s, NEO_S_OUTKBS, NEO_S_ERROR)) {
      printf("    ???");
    } else if (neo_stats_get_generic(s, NEO_S_OUTKBS, NEO_S_UNAVAIL)) {
      printf("      -");
    } else {
      printf(" %6i", neo_stats_get_generic(s, NEO_S_OUTKBS, NEO_S_INT));
    }

    if (neo_stats_get_generic(s, NEO_S_TOTALPPS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_TOTALPPS, NEO_S_UNAVAIL)) {
      printf("     -");
    } else {
      printf(" %5i", neo_stats_get_generic(s, NEO_S_TOTALPPS, NEO_S_INT));
    }

    if (neo_stats_get_generic(s, NEO_S_INPPS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_INPPS, NEO_S_UNAVAIL)) {
      printf("     -");
    } else {
      printf(" %5i", neo_stats_get_generic(s, NEO_S_INPPS, NEO_S_INT));
    }
    
    if (neo_stats_get_generic(s, NEO_S_OUTPPS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_OUTPPS, NEO_S_UNAVAIL)) {
      printf("     -");
    } else {
      printf(" %5i", neo_stats_get_generic(s, NEO_S_OUTPPS, NEO_S_INT));
    }

    if (neo_stats_get_generic(s, NEO_S_INERRPS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_INERRPS, NEO_S_UNAVAIL)) {
      printf("       -");
    } else {
      printf("  %4i", neo_stats_get_generic(s, NEO_S_INERRPS, NEO_S_INT));
    }
    if (neo_stats_get_generic(s, NEO_S_OUTERRPS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_OUTERRPS, NEO_S_UNAVAIL)) {
      printf("       -");
    } else {
      printf("  %4i", neo_stats_get_generic(s, NEO_S_OUTERRPS, NEO_S_INT));
    }

    printf("\n");
  }
}

void neo_command_stats_print_boardstats(neo_boardlist *bl)
{
  int i, j;
  neo_board *b;
  neo_stats *s;
  
  printf("Board statistics:\n");
  printf(" b type           sg   ul ultype       kbs  ikbs  okbs   pps  ipps  opps\n");
  printf("------------------------------------------------------------------------\n");
  j=neo_boardlist_get_size(bl);
  for (i=0; i<j; i++) {
    b=neo_boardlist_get_element(bl, i);
    s=neo_board_get_stats(b);
    printf("%2i %-13.13s ", neo_board_get_number(b), neo_board_get_descr(b));

    if (!neo_board_is_present(b)) {
      printf("\n");
      continue;
    }
    
    printf("  %1i ", neo_board_get_segment(b));

    if (neo_board_link_is_up(b)) {
      printf("%4i ", neo_board_get_speed(b)/1000000);
    } else if (neo_board_link_is_down(b)) {
      printf("     ");
    } else if (neo_board_link_is_unknown(b)){
      printf("???? ");
    }

    printf("%-10.10s ", neo_board_get_uplinkdescr(b));

    if (neo_stats_get_generic(s, NEO_S_TOTALKBS, NEO_S_ERROR)) {
      printf("  ???");
    } else if (neo_stats_get_generic(s, NEO_S_TOTALKBS, NEO_S_UNAVAIL)) {
      printf("    -");
    } else {
      printf("%5i", neo_stats_get_generic(s, NEO_S_TOTALKBS, NEO_S_INT));
    }
    
    if (neo_stats_get_generic(s, NEO_S_INKBS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_INKBS, NEO_S_UNAVAIL)) {
      printf("     -");
    } else {
      printf(" %5i", neo_stats_get_generic(s, NEO_S_INKBS, NEO_S_INT));
    }
    
    if (neo_stats_get_generic(s, NEO_S_OUTKBS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_OUTKBS, NEO_S_UNAVAIL)) {
      printf("     -");
    } else {
      printf(" %5i", neo_stats_get_generic(s, NEO_S_OUTKBS, NEO_S_INT));
    }

    if (neo_stats_get_generic(s, NEO_S_TOTALPPS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_TOTALPPS, NEO_S_UNAVAIL)) {
      printf("     -");
    } else {
      printf(" %5i", neo_stats_get_generic(s, NEO_S_TOTALPPS, NEO_S_INT));
    }

    if (neo_stats_get_generic(s, NEO_S_INPPS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_INPPS, NEO_S_UNAVAIL)) {
      printf("     -");
    } else {
      printf(" %5i", neo_stats_get_generic(s, NEO_S_INPPS, NEO_S_INT));
    }

    if (neo_stats_get_generic(s, NEO_S_OUTPPS, NEO_S_ERROR)) {
      printf("   ???");
    } else if (neo_stats_get_generic(s, NEO_S_OUTPPS, NEO_S_UNAVAIL)) {
      printf("     -");
    } else {
      printf(" %5i", neo_stats_get_generic(s, NEO_S_OUTPPS, NEO_S_INT));
    }

    printf("\n");
  }
}
