#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include "neo.h"

int neo_location_create_from_string(neo_location *loc, char *in)
{
  char *atptr;
  char bpbuff[1024];

  loc->instring=strdup(in);

  atptr=strchr(in, '@');
  if (atptr && atptr!=in) {
    /* there is a port or board/port */
    strncpy(bpbuff, in, atptr-in);
    bpbuff[atptr-in]='\0';
    neo_boardport_create_from_string(&(loc->bp), bpbuff);
  } else {
    neo_boardport_create_null(&(loc->bp));
  }

  /* if there's no at sign assume a simple host */  
  if (atptr==NULL) atptr=in;

  /* now get the devices */
  loc->devicelist=neo_devicelist_create_from_string(atptr);
  if (!loc->devicelist) {
    neo_error=NEO_ERR_BADFORMAT;
    return(-1);
  }
  return(0);
}

int neo_location_create_from_boardport_and_device(neo_location *loc, neo_boardport *bp, char *devicename)
{
  char *bpstr;
  
  loc->instring=malloc(1000);
  bpstr=neo_boardport_to_string(bp);
  sprintf(loc->instring, "%s@%s", bpstr, devicename);

  neo_boardport_copy(&(loc->bp), bp);
  loc->devicelist=neo_devicelist_create_from_string(devicename);
  if (!loc->devicelist) {
    neo_error=NEO_ERR_BADFORMAT;
    return(-1);
  }
  return(0);
}

int neo_location_num_hosts(neo_location *loc)
{
  return(neo_devicelist_get_size(loc->devicelist));
}

neo_device *neo_location_get_device_n(neo_location *loc, int n)
{
  return(neo_devicelist_get_element(loc->devicelist, n));
}

neo_device *neo_location_get_first_device(neo_location *loc)
{
  return(neo_devicelist_get_element(loc->devicelist, 0));
}

int neo_location_is_simplehost(neo_location *loc)
{
  if ((neo_location_num_hosts(loc) == 1) &&
      (!neo_location_has_board(loc)) &&
      (!neo_location_has_port(loc))) {
    return(1);
  }
  return(0);
}

int neo_location_has_port(neo_location *loc)
{
  return(neo_boardport_has_port(&(loc->bp)));
}

int neo_location_has_board(neo_location *loc)
{
  return(neo_boardport_has_board(&(loc->bp)));
}

int neo_location_star_port(neo_location *loc)
{
  return(neo_boardport_star_port(&(loc->bp)));
}

int neo_location_star_board(neo_location *loc)
{
  return(neo_boardport_star_board(&(loc->bp)));
}

int neo_location_get_portnum(neo_location *loc)
{
  return(neo_boardport_get_portnum(&(loc->bp)));
}

int neo_location_get_boardnum(neo_location *loc)
{
  return(neo_boardport_get_boardnum(&(loc->bp)));
}

char *neo_location_get_instr(neo_location *loc)
{
  return(loc->instring);
}

char *neo_location_to_string(neo_location *loc)
{
  /* THIS SHOULD CHANGE */
  return(loc->instring);
}

int neo_location_set_deviceconsts(neo_location *loc, neo_global *g)
{
  int i, j, ret, finalret;

  finalret=0;
  j=neo_location_num_hosts(loc);
  for (i=0; i<j; i++) {
    ret=neo_device_set_const_from_probe(neo_location_get_device_n(loc, i), g);
    if (ret==-1) {
      printf("Could not contact device %s\n", neo_device_get_name(neo_location_get_device_n(loc, i)));
      finalret=-1;
    }
  }
  return(finalret);
}

/* this copies pointers to the device list */
void neo_location_copy(neo_location *src, neo_location *dst)
{
  memcpy(dst, src, sizeof(neo_location));
}

/* Copy boardport info into bp, which should already
 * be initialized
 */
void neo_location_extract_boardport(neo_location *loc, neo_boardport *bp)
{
  neo_boardport_copy(bp, &(loc->bp));
}

int neo_location_debugprint(neo_location *loc)
{
  char *bp;

  bp=neo_boardport_to_string(&(loc->bp));
  printf("boardport: %s\n", bp);
  free(bp);
  if (loc->devicelist) {
    neo_devicelist_debugprint(loc->devicelist);
  }
  return(0);
}

int neo_location_is_equal(neo_location *a, neo_location *b)
{
  if (!neo_boardport_is_equal(&(a->bp), &(b->bp))) return(0);
  if (!neo_devicelist_is_equal(a->devicelist, b->devicelist)) return(0);
  return(1);
}

void neo_location_free(neo_location *loc)
{
  if (loc==NULL) return;
  neo_boardport_free(&(loc->bp));
  neo_devicelist_free_all(loc->devicelist);
  free(loc->instring);
}
